@@ -524,7 +524,7 @@ class AugmentedPath(pathlib.Path):
                 # python-magic
                 return magic.from_file(self.as_posix(), mime=True)
 
-    def checksum(self, cypher=default_cypher):
+    def checksum(self, cypher=default_cypher, extra_cyphers=tuple()):
         """ checksum() always recomputes from the data
             meta.checksum is static for cache and remote IF it exists """
 
@@ -539,10 +539,16 @@ class AugmentedPath(pathlib.Path):
                 cypher = self.cypher
 
             m = cypher()
+            extra = [c() for c in extra_cyphers]
             for chunk in self.data:
                 m.update(chunk)
+                for me in extra:
+                    me.update(chunk)
 
-            return m.digest()
+            if extra_cyphers:
+                return tuple(_.digest() for _ in (m, *extra))
+            else:
+                return m.digest()
 
     def copy_to(self, target, force=False):
         """ copy from a the current path object to a target path """
@@ -563,11 +569,13 @@ class AugmentedPath(pathlib.Path):
         if not target.is_dir():
             raise NotADirectoryError(f'{target} is not a directory')
 
-        self.copy_to(target / self.name)
+        target_file = target / self.name
+        self.copy_to(target_file, force=force)
+        return target_file
 
     def copy_infrom(self, source, force=False):
         """ copy into the current directory a file from somewhere else """
-        source.copy_outto(self, force=force)
+        return source.copy_outto(self, force=force)
 
 
 class AugmentedPathPosix(AugmentedPath, pathlib.PosixPath): pass
@@ -700,6 +708,11 @@ class LocalPath(EatPath, AugmentedPath):
 
         return self._cache
 
+    @property
+    def cache_id(self):
+        """ abstraction violating but fast way to get cache.id """
+        raise NotImplementedError('implement in subclass')
+
     def cache_init(self, id_or_meta, anchor=False):
         """ wow it took way too long to realize this was the way to do it >_<
             **kwargs are passed to _cache_class and _remote_class.init """
